from plone.app.contentmenu.menu import WorkflowMenu as BaseWorkflowMenu

from raptus.workflownotification.interfaces import INotificationEnabled

class WorkflowMenu(BaseWorkflowMenu):
    
    def getMenuItems(self, context, request):
        items = super(WorkflowMenu, self).getMenuItems(context, request)
        notification = INotificationEnabled(context, None)
        if notification is None:
            return items
        
        transitions = notification.transitions()
        
        for item in items:
            if item['extra']['id'][20:] in transitions:
                item['extra']['class'] = '%s workflownotification' % item['extra']['class']
                item['action'] = item['action'].replace('content_status_modify', '@@workflownotification_form')
                
        return items



from Acquisition import aq_base, aq_inner, aq_parent
from AccessControl import Unauthorized

from zope.interface import implements, Interface, classProvides
from zope.component import getMultiAdapter
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope import schema
from zope.i18n import translate
from zope.formlib import form
from zope.app.form.browser.interfaces import ISourceQueryView

from plone.app.vocabularies.users import UsersSource as BaseUsersSource, UsersSourceQueryView as BaseUsersSourceQueryView
from plone.app.vocabularies.groups import GroupsSource as BaseGroupsSource, GroupsSourceQueryView as BaseGroupsSourceQueryView

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _p

from Products.statusmessages.interfaces import IStatusMessage

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser import BrowserView
from Products.Five.formlib import formbase

from raptus.workflownotification.interfaces import INotificationForm, INotificationSender
from raptus.workflownotification.utils import UserRecipientFactory
from raptus.workflownotification import workflownotificationMessageFactory as _

class UsersSource(BaseUsersSource):
    classProvides(IContextSourceBinder)
    def search(self, query):
        try:
            return super(UsersSource, self).search(query.encode('utf-8'))
        except:
            return super(UsersSource, self).search(query)
        
class UsersSourceQueryView(BaseUsersSourceQueryView):
    def getTerm(self, value):
        user = self.context.get(value)
        token = value
        title = value
        if user is not None:
            title = user.getProperty('fullname', None) or user.getId()
        return SimpleTerm(value, token=token, title=title.decode('utf-8'))

class GroupsSource(BaseGroupsSource):
    classProvides(IContextSourceBinder)
    def search(self, query):
        try:
            return super(GroupsSource, self).search(query.encode('utf-8'))
        except:
            return super(GroupsSource, self).search(query)
        
class GroupsSourceQueryView(BaseGroupsSourceQueryView):
    def getTerm(self, value):
        user = self.context.get(value)
        token = value
        title = value
        if user is not None:
            title = user.getProperty('title', None) or user.getId()
        return SimpleTerm(value, token=token, title=title.decode('utf-8'))

class INotificationForm(Interface):
    """Define the fields of our form
    """

    message = schema.Text(title=_(u"Message"),
                          required=True)
    
    users = schema.List(title=_(u"Users"),
                        required=False,
                        value_type=schema.Choice(source=UsersSource))
    
    groups = schema.List(title=_(u"Groups"),
                         required=False,
                         value_type=schema.Choice(source=GroupsSource))
    
class NotificationSearch(BrowserView):
    
    def __call__(self, f, q):
        if not INotificationForm.providedBy(self.context):
            return
        
        field = self.context.form_fields[f].field.bind(self.context.context)
        source = field.value_type.source
        query_view = getMultiAdapter((source, self.request), ISourceQueryView)
        
        if q:
            terms = set([query_view.getTerm(token) for token in source.search(q)])
        else:
            terms = set()
        
        return '\n'.join(["%s|%s" % (t.token,  t.title or t.token)
                            for t in sorted(terms, key=lambda t: t.title)])
        
class NotificationForm(formbase.PageForm):
    implements(INotificationForm)
    form_fields = form.FormFields(INotificationForm)
    label = _(u"Send notifications")
    workflow_action = None

    template = ViewPageTemplateFile('notification_form.pt')
    template_standalone = ViewPageTemplateFile('notification_form_standalone.pt')
    
    def __call__(self, workflow_action, standalone=False):
        self.workflow_action = workflow_action
        self.standalone = standalone
        self.request.set('disable_border', 1)
        return super(NotificationForm, self).__call__()

    def update(self):
        wftool = getToolByName(aq_inner(self.context), 'portal_workflow')
        transitions = [t['id'] for t in wftool.getTransitionsFor(aq_inner(self.context))]
        if not self.workflow_action in transitions:
            raise Unauthorized
        
        if self.standalone:
            self.template = self.template_standalone
            
        self.utils = getMultiAdapter((self.context, self.request), name=u'workflownotification')
        self.recipients = self.utils.getRecipientsFor(self.context, self.workflow_action)
        self.message_template = self.utils.getTemplateFor(self.context, self.workflow_action)
        
        actions = form.Actions()
        for name, a in self.actions.byname.items():
            if name == 'form.actions.label_send':
                a.label = _p(wftool.getTitleForTransitionOnType(self.workflow_action, self.context.portal_type))
            actions.append(a)
        self.actions = actions
        
        self.form_fields.get('message').field.default = self.message_template
        
        super(NotificationForm, self).update()
    
    def js(self):
        url = self.context.absolute_url()
        js = []
        for id in ('users', 'groups',):
            js.append(self.js_template % dict(id=id,
                                              url=url))
        return '\n'.join(js)

    @form.action(_p(u"label_send", default=u"Send"))
    def action_send(self, action, data):
        """Send the notification
        """
        sender = INotificationSender(self.context)
        recipients = self.recipients
        for userid in data['users']:
            recipients.append(UserRecipientFactory(userid))
        groups = getToolByName(aq_inner(self.context), 'portal_groups')
        for groupid in data['groups']:
            group = groups.getGroupById(groupid)
            users = group.getGroupMemberIds()
            for userid in users:
                recipients.append(UserRecipientFactory(userid))
        receivers, message = sender.send(self.workflow_action, data['message'], recipients)
        receivers = ', '.join([r.name for r in receivers])
        
        statusmessages = IStatusMessage(self.request)
        statusmessages.addStatusMessage(_('message_success', default='Notifications successfully sent to ${receivers}', mapping={'receivers': receivers}), type='info')
        
        return self.context.content_status_modify(workflow_action=self.workflow_action, comment='', notified=True)

    @form.action(_p(u"label_cancel", default=u"Cancel"),validator=lambda *args, **kwargs: {})
    def action_cancel(self, action, data):
        """Cancel
        """
        return self.request.response.redirect(self.context.absolute_url())
    
    js_template = """\
    (function($) {
        $().ready(function() {
            $('#formfield-form-%(id)s').each(function() {
                if(!$('#formfield-form-%(id)s #%(id)s-input').length)
                    $('#formfield-form-%(id)s').append('<input name="%(id)s-input" type="text" id="%(id)s-input" />');
                $('#formfield-form-%(id)s .queries').remove();
                $('#formfield-form-%(id)s #%(id)s-input').autocomplete('%(url)s/@@workflownotification_form/@@workflownotification_form-search?f=%(id)s', {
                    autoFill: false,
                    minChars: 2,
                    max: 10,
                    mustMatch: false,
                    matchContains: true,
                    formatItem: function(row, idx, count, value) { return row[1]; },
                    formatResult: function(row, idx, count) { return ""; }
                }).result(
                    function(event, data, formatted) {
                        var field = $('#formfield-form-%(id)s input[type="checkbox"][value="' + data[0] + '"]');
                        if(field.length == 0)
                            $('#formfield-form-%(id)s #%(id)s-input').before("<" + "label class='plain'><" + "input type='checkbox' name='form.%(id)s:list' checked='checked' value='" + data[0] + "' /> " + data[1] + "</label><br />");
                        else
                            field.each(function() { this.checked = true });
                        if(data[0])
                            $('#formfield-form-%(id)s #%(id)s-input').val('');
                    });
            })
        });
    })(jQuery);
    """



from zope.i18nmessageid import MessageFactory
workflownotificationMessageFactory = MessageFactory('raptus.workflownotification')
from Products.PlacelessTranslationService.utility import PTSTranslationDomain
workflownotificationdomain = PTSTranslationDomain('raptus.workflownotification')


from zope.component import queryAdapter
from zope.interface import implements

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from raptus.workflownotification.interfaces import INotificationEnabled, INotificationRecipientProvider, INotificationTemplateProvider, INotificationUtils

class WorkflownotificationView(BrowserView):
    """ View providing utility methods concerning workflownotifications
    """
    implements(INotificationUtils)
    
    def getEnabledTransitionsFor(self, objs):
        """ returns all transitions which have notifications enabled for the given objects
        """
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        if not type(objs) is type([]):
            objs = [objs,]
        transitions = []
        for obj in objs:
            if type(obj) is type(''):
                obj = portal.restrictedTraverse(obj)
            enabler = INotificationEnabled(obj, None)
            if enabler:
                transitions.extend(enabler.transitions())
        return transitions
    
    def getWorkflowFor(self, obj):
        """ returns the workflow_id for the given object
        """
        wftool = getToolByName(obj, 'portal_workflow')
        chain = wftool.getChainFor(obj)
        return chain[0]
    
    def getRecipientsFor(self, obj, action):
        """ returns a list of default recipients for the given object and action
        """
        recipients = []
        workflow = self.getWorkflowFor(obj)
        provider_global = queryAdapter(obj, INotificationRecipientProvider)
        if provider_global is not None:
            recipients.extend(provider_global.recipients(workflow, action))
        
        provider_workflow = queryAdapter(obj, INotificationRecipientProvider, name=workflow)
        if provider_workflow is not None:
            recipients.extend(provider_workflow.recipients(workflow, action))
        
        provider_transition = queryAdapter(obj, INotificationRecipientProvider, name='%s.%s' % (workflow, action))
        if provider_transition is not None:
            recipients.extend(provider_transition.recipients(workflow, action))
            
        return recipients
    
    def getTemplateFor(self, obj, action):
        """ returns the template for the given object and action
        """
        workflow = self.getWorkflowFor(obj)
        provider_transition = queryAdapter(obj, INotificationTemplateProvider, name='%s.%s' % (workflow, action))
        if provider_transition is not None:
            return provider_transition.template(workflow, action)
        
        provider_workflow = queryAdapter(obj, INotificationTemplateProvider, name=workflow)
        if provider_workflow is not None:
            return provider_workflow.template(workflow, action)
            
        provider_global = queryAdapter(obj, INotificationTemplateProvider)
        if provider_global is not None:
            return provider_global.template(workflow, action)
            
        return u""



from zope.component import adapts, getMultiAdapter
from zope.interface import implements, Interface
from zope.i18n import translate

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _p
from Products.CMFPlone.utils import safe_unicode

from interfaces import INotificationSender
from raptus.workflownotification import workflownotificationMessageFactory as _

class WorkflownotificationSender(object):
    adapts(Interface)
    implements(INotificationSender)
    
    def __init__(self, context):
        self.context = context
        
    def send(self, action, message, recipients):
        """ send the notifications
        """
        mailhost = getToolByName(self.context, 'MailHost')
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        mfrom = '%s <%s>' % (portal.getProperty('email_from_name', ''), portal.getProperty('email_from_address', ''))
        subject = translate(_('email_subject', default='New notification from ${site}', mapping={'site': portal.getProperty('title', '')}), context=self.context.request)
        wftool = getToolByName(self.context, 'portal_workflow')
        utils = getToolByName(self.context, 'plone_utils')
        mship = getToolByName(self.context, 'portal_membership')
        wnutils = getMultiAdapter((self.context, self.context.request), name=u'workflownotification')
        workflow = wftool.getWorkflowById(wnutils.getWorkflowFor(self.context))
        tdef = workflow.transitions.get(action, None)
        if not tdef:
            return
        issuer = mship.getMemberInfo()
        if not issuer:
            issuer = {'username': 'Anonymous'}
        replacement = dict(name='',
                           title=self.context.Title(),
                           url=self.context.absolute_url(),
                           old_state=translate(_p(utils.getReviewStateTitleFor(self.context)), context=self.context.request),
                           new_state=translate(_p(wftool.getTitleForStateOnType(tdef.new_state_id, self.context.portal_type)), context=self.context.request),
                           transition=translate(_p(tdef.actbox_name), context=self.context.request),
                           issuer=safe_unicode(issuer.get('fullname', issuer['username'])))
        anon_message = message % replacement
        sent = []
        receivers = []
        for r in recipients:
            replacement.update(dict(name=r.name))
            msg = message % replacement
            if not r.email in sent:
                mailhost.secureSend(msg,
                                    mto=r.email,
                                    mfrom=mfrom,
                                    subject=subject,
                                    charset='utf-8')
                sent.append(r.email)
                receivers.append(r)
        return receivers, anon_message
    


from zope.interface import implements
from zope.app.component.hooks import getSite

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode

from interfaces import INotificationRecipient, INotificationEnabled

class NotificationRecipient(object):
    implements(INotificationRecipient)
    
    def __init__(self, name, email):
        self.email = email
        self.name = safe_unicode(name)
        
    def __str__(self):
        return '<NotificationRecipient %s <%s>>' % (self.name, self.email)

def UserRecipientFactory(userid):
    context = getSite()
    mtool = getToolByName(context, 'portal_membership')
    member = mtool.getMemberById(userid)
    mdata = getToolByName(context, 'portal_memberdata')
    member = mdata.wrapUser(member)
    return NotificationRecipient(member.getProperty('fullname', member.getUserName()), member.getProperty('email', None))


from zope.interface import Interface, Attribute

class INotificationUtils(Interface):
    """ View providing utility methods concerning workflownotifications
    """
    
    def getEnabledTransitionsFor(objs):
        """ returns all transitions which have notifications enabled for the given objects
        """
    
    def getWorkflowFor(obj):
        """ returns the workflow_id for the given object
        """
    
    def getRecipientsFor(obj, action):
        """ returns a list of default recipients for the given object and action
        """
    
    def getTemplateFor(obj, action):
        """ returns the template for the given object and action
        """
        
class INotificationSender(Interface):
    """
    """
    
    def send(action, message, recipients):
        """ send the notifications
        """

class INotificationForm(Interface):
    """ Marker interface for notification form
    """

class INotificationEnabled(Interface):
    """
    """
    
    def transitions():
        """ returns a list of transition names for which notifications are sent (None for all)
        """

class INotificationRecipient(Interface):
    """ A recipient
    """
    name = Attribute('name')
    email = Attribute('email')

class INotificationRecipientProvider(Interface):
    """
    """
    
    def recipients(workflow, action):
        """ returns a list of recipients (objects implementing INotificationRecipient) to be notified
        """
        
class INotificationTemplateProvider(Interface):
    """
    """
    
    def template(workflow, action):
        """ returns a template to be used as the message
            
            the following keywords may be used in the form of %(keyword)s by the corresponding string:
            
            name:       the full name of the receiver
            url:        the url to the content object
            title:      the title of the content object
            old_state:  the old state of the content object
            new_state:  the new state of the content object
            transition: the transition made on the the content object
            issuer:     the full name of the issuer
        """



import os
from Acquisition import Implicit, aq_inner
from urlparse import urlsplit

from zope.component import getMultiAdapter
from zope.interface import implements

from kss.core import kssaction

from plone.app.layout.globals.interfaces import IViewView
from plone.app.kss.interfaces import IPloneKSSView
from plone.app.kss.plonekssview import PloneKSSView

class NotificationFormView(Implicit, PloneKSSView):
    implements(IPloneKSSView, IViewView)
    
    @kssaction
    def displayWorkflowNotificationForm(self, url):
        ksscore = self.getCommandSet('core')

        (proto, host, path, query, anchor) = urlsplit(url)
        action = query.split("workflow_action=")[-1].split('&')[0]
        context = self.context.restrictedTraverse(os.path.dirname(path))
        self.request.URL = '%s/@@workflownotification_form' % context.absolute_url()
        form = getMultiAdapter((context, self.request), name=u'workflownotification_form')(workflow_action=action, standalone=True)
        selector = ksscore.getCssSelector('#contentActionMenus')
        ksscore.setStyle(selector, 'display', 'none')
        selector = ksscore.getCssSelector('#kss-workflownotification_form, #kss-workflownotification_form-overlay')
        ksscore.deleteNode(selector)
        selector = ksscore.getCssSelector('body')
        ksscore.insertHTMLAsLastChild(selector, '%s<div id="kss-workflownotification_form-overlay"></div>' % form.replace('class="workflownotification_form"', 'class="workflownotification_form" id="kss-workflownotification_form"'))
        self.cancelRedirect()
        
    @kssaction
    def removeWorkflowNotificationForm(self):
        ksscore = self.getCommandSet('core')
        
        selector = ksscore.getCssSelector('#contentActionMenus')
        ksscore.setStyle(selector, 'display', 'block')
        selector = ksscore.getCssSelector('#kss-workflownotification_form, #kss-workflownotification_form-overlay')
        ksscore.deleteNode(selector)
        self.cancelRedirect()
        